@@ -65,17 +65,18 @@ class Message(object):
         return self.sender
     def lazy_set_property(self, name, value):
         '''this does same as set_property, without validation'''
-        self._properties[name] = self.pack_property(name, value)        
+        self._properties[name] = value
     def set_property(self, name, value):
         '''set required property of the message'''
         if name not in self.properties:
             raise InvalidMessageProperty('Invalid Message Property: %s' % name)
-        self._properties[name] = self.pack_property(name, value)
+        self._properties[name] = value
     def get_property(self, name, default=None):
+        '''get a given property from the message, if the retrieved property is None, ``default`` is returned'''
         if name not in self.properties:
             raise InvalidMessageProperty('Invalid Message Property: %s' % name)
         if not self._properties[name] is None:
-            return self.unpack_property(name, self._properties[name])
+            return self._properties[name]
         else:
             return default
     def pack_property(self, name, value):
@@ -212,11 +213,11 @@ class MessageManager(util.ProcessLocalSingleton):
         if not self.validate_type(msg.message_type):
             return False
         # Here we need to gracefully handle messages of type that isn't subscribed by any receivers
-        if not self.message_receiver_map.has_key(msg.message_type) and not self.message_receiver_map[WildCardMessageType]:
-            return False
-        else:
-            self.active_queue.append(msg)
-            return True
+        # if not self.message_receiver_map.has_key(msg.message_type) and not self.message_receiver_map[WildCardMessageType]:
+        #     return False
+        # else:
+        self.active_queue.append(msg)
+        return True
     def trigger(self, msg):
         '''
         same as queue_message, except that this is synchronous
