@@ -1,28 +1,29 @@
-'''pysage is a high-level message passing library with currency in mind.
+# pysage is a high-level message passing library with currency in mind.
+# 
+# For more information: http://code.google.com/p/pysage/
+# 
+# Copyright (c) 2007-2008 Shuo Yang (John) <bigjhnny@gmail.com>
+# 
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+# 
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+# 
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+# THE SOFTWARE.
 
-For more information: http://code.google.com/p/pysage/
+'''implementation of actors, manager and grouping/network functionality'''
 
-Copyright (c) 2007-2008 Shuo Yang (John) <bigjhnny@gmail.com>
-
-Permission is hereby granted, free of charge, to any person obtaining a copy
-of this software and associated documentation files (the "Software"), to deal
-in the Software without restriction, including without limitation the rights
-to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-copies of the Software, and to permit persons to whom the Software is
-furnished to do so, subject to the following conditions:
-
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-THE SOFTWARE.
-'''
-# system.py
 import struct
 import messaging
 import transport
@@ -30,6 +31,7 @@ import util
 import time
 import process as processing
 import logging
+import warnings
 
 __all__ = ('Message', 'ActorManager', 'Actor', 'PacketError', 'PacketTypeError', 'GroupAlreadyExists', 'GroupDoesNotExist', 'CreateGroupError',
            'DefaultActorFailed', 'GroupFailed', 'get_logger', 'WrongMessageTypeSpecified')
@@ -292,6 +294,8 @@ class ActorManager(messaging.MessageManager):
         self.transport = transport_class()
         self.transport.connect(host, port)
         return self
+    def disconnect(self):
+        self.transport.disconnect()
     def send_message(self, msg, address=None):
         '''
         send a message to a network
@@ -300,6 +304,8 @@ class ActorManager(messaging.MessageManager):
             - `msg`: the message to send
             - `clientid`: the network for which to send the message to
         '''
+        if not type(msg).packet_type:
+            raise PacketTypeError('Packet_type must be specified by class "%s"' % type(msg))
         self.transport.send(msg.to_string(), address=address)
         return self
     def queue_message_to_group(self, group, msg):
@@ -310,6 +316,8 @@ class ActorManager(messaging.MessageManager):
         self.log(logging.INFO, 'queuing message "%s" to "%s"' % (msg, _clientid))
         self.ipc_transport.send(msg.to_string(), _clientid)
     def broadcast_message(self, msg):
+        if not type(msg).packet_type:
+            raise PacketTypeError('Packet_type must be specified by class "%s"' % type(msg))
         self.transport.send(msg.to_string(), broadcast=True)
         return self
     def packet_handler(self, packet, address):
@@ -327,11 +335,16 @@ class ActorManager(messaging.MessageManager):
         if packet_class.__name__ == 'Message':
             return
         if not packet_class.packet_type:
-            raise PacketTypeError('Packet_type must be specified by class "%s"' % packet_class)
+            # raise PacketTypeError('Packet_type must be specified by class "%s"' % packet_class)
+            return
         if packet_class.packet_type <= 100:
             raise PacketTypeError('Packet_type must be greater than 100.  Had "%s"' % packet_class.packet_type)
         if self.packet_types.has_key(packet_class.packet_type):
-            raise PacketTypeError('Packet_type is already registered with packet "%s"' % self.packet_types[packet_class.packet_type])
+            if packet_class.__name__ == self.packet_types[packet_class.packet_type].__name__:
+                warnings.warn('Attempted to register same message class %s.  Skipped.' % packet_class, stacklevel=3)
+                return
+            else:
+                raise PacketTypeError('Trying to register %s.  But packet_type %s is already registered with packet "%s"' % (packet_class, packet_class.packet_type, self.packet_types[packet_class.packet_type]))
         self.packet_types[packet_class.packet_type] = packet_class
         self.message_map[packet_class.__name__] = packet_class
     def validate_groups_mode(self):
@@ -452,7 +465,11 @@ class Message(messaging.Message):
                         buf = self.pack_attr(_type[j], buf, v, name)
             # for mono types, just pack it
             else:
-                buf = self.pack_attr(_type, buf, value, name)
+                pack_func = getattr(self, 'pack_' + name, None)
+                if pack_func:
+                    buf = self.pack_attr(_type, buf, pack_func(value), name)
+                else:
+                    buf = self.pack_attr(_type, buf, value, name)
         return buf
     def from_string(self, data):
         '''unpacks the property data into the object, from binary stream'''
@@ -477,9 +494,13 @@ class Message(messaging.Message):
                     self.set_property(name, values)
             # if not composite, just unpack them and set the property
             else:
+                unpack_func = getattr(self, 'unpack_' + name, None)
                 value, size = self.unpack_attr(_type, data, pos)
                 pos += size
-                self.set_property(name, value)
+                if unpack_func:
+                    self.set_property(name, unpack_func(value))
+                else:
+                    self.set_property(name, value)
         if pos != len(data):
             raise PacketError('incorrect length upon unpacking %s: got %i expected %i' % (self.__class__.__name__, len(data), pos))
         return self
